package com.flyfish.framework.service;

import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.repository.ReactiveUserRepository;
import com.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import com.flyfish.framework.utils.ReactiveRedisOperations;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;

/**
 * 异步用户service
 *
 * @author wangyu
 */
@Service
public class ReactiveUserService extends BaseReactiveServiceImpl<User> {

    @Resource
    private ReactiveRedisOperations reactiveRedisOperations;

    /**
     * 获取用户数据
     *
     * @param username 用户
     * @return 结果
     */
    public Mono<User> findByUsername(String username) {
        String key = getCacheKey(username);
        return reactiveRedisOperations.hasKey(key).flatMap(exists -> exists ? reactiveRedisOperations.get(key) :
                ((ReactiveUserRepository) repository).findByUsername(username));
    }

    /**
     * 获取缓存键
     *
     * @param username 用户名
     * @return 结果
     */
    private String getCacheKey(String username) {
        return "user-" + username;
    }

}
